# -*- coding: utf8 -*-
from django.core import validators
from django import db
from django.forms import Field, widgets, ValidationError

from .models import PortailUser

import re


class PortailUserWidget(widgets.Input):
    input_type = "text"
    template_name = "form_widgets/portail_user.html"

    def render(self, name, value, attrs=None, renderer=None):
        if value is None:
            value = ""

        if type(value) == int:
            # Clé primaire d’un PortailUser
            try:
                value = PortailUser.objects.get(id=value).username
            except PortailUser.DoesNotExist:
                value = ""

        attrs["class"] = "typeahead form-control"

        return super(PortailUserWidget, self).render(name, value, attrs)

    class Media:
        js = ("js/typeahead.bundle.min.js", "js/typeahead_config.js")


class PortailUserField(Field):
    default_error_messages = {"no_such_user": "Cet utilisateur n’existe pas."}

    widget = PortailUserWidget

    def to_python(self, value):
        if value in validators.EMPTY_VALUES:
            return None

        if isinstance(value, PortailUser):
            return value

        if isinstance(value, str):
            value = value.lower()

        if not isinstance(value, str):
            return None

        try:
            return PortailUser.objects.get_from_ldap(value)
        except PortailUser.DoesNotExist:
            raise ValidationError(self.default_error_messages["no_such_user"])


class PortailUserPhoneField(Field):
    error = "Ce numéro de téléphone est invalide."

    def __init__(self, *args, **kwargs):
        if "max_length" in kwargs:
            del kwargs["max_length"]
        self.required = kwargs.get("required", True)
        Field.__init__(self, *args, **kwargs)

    def clean(self, value):
        if not value and not self.required:
            return value

        value = value.replace(" ", "")
        value = value.replace(".", "")

        if not re.match("^[0-9+]+$", value):
            raise ValidationError(PortailUserPhoneField.error)

        if value[0:2] == "00":
            value = "+" + value[2:]

        if value[0] == "0":
            if len(value) != 10:
                raise ValidationError(PortailUserPhoneField.error)
            value = "+33" + value[1:]

        return value


class PortailUserDbPhoneField(db.models.CharField):
    def __init__(self, *args, **kwargs):
        db.models.CharField.__init__(self, *args, **kwargs)

    def get_internal_type(self):
        return "CharField"

    def formfield(self, **kwargs):
        defaults = {"form_class": PortailUserPhoneField}
        defaults.update(kwargs)
        return super(PortailUserDbPhoneField, self).formfield(**defaults)
